#include <iostream>
#include <fstream>
#include <vector>
#include <cmath>
#include <algorithm>

using rvec = std::vector<double>;

// ...

/* Adams Bashforth numerical step which computes
 * y^{n+1} = y^n + 3/2*dt*f(t^n,y^n) - 1/2* dt*f(t^{n-1},y^{n-1})
 * in order to solve y' = f(t,y)
 * Vectors yNP1, yN and yNM1 are supposed to have the same size.
 */
void oneStepAdamsBashforth2(rvec& yNP1, rvec& yN, rvec& yNM1, double (*f) (double, double), double dt, double tN) {

  for (int i=0; i<yNP1.size(); i++)
    yNP1[i] = yN[i] + dt*1.5*f(tN,yN[i]) - 0.5*dt*f(tN-dt,yNM1[i]); 

};

/* Forward Euler step
 * y^{n+1} = y^n + dt*f(t^n,y^n)
 * in order to solve y' = f(t,y)
 * Vectors yNP1 and yN are supposed to have the same size.
 */
void oneStepEuler(rvec& yNP1, rvec& yN, double (*f) (double, double), double dt, double tN) {

  for (int i=0; i<yNP1.size(); i++)
    yNP1[i] = yN[i] + dt*f(tN,yN[i]); 

};

int main() {

  int N = 1000;
  // Data vector at different step
  rvec y0(N),y1(N),y2(N);
  // Points
  rvec x(N);

  // Problem times
  double dt     = 0.01;
  double t      = 0.;
  double tFinal = 1.;

  // x_i and Initial condition
  for (int i=0; i<N; i++) {
    x [i] = i/(double)(N-1);
    y0[i] = initCond(x[i]);
  }

  // First step is performed with Euler scheme
  oneStepEuler(y1,y0,&fTest,dt,t);
  t += dt;
 
  // These scalars will be used in the loop to compute the L2 norms
  // of the solution and the difference with approx at all times
  double normSol(0.e0),normDiff(0.e0);

  // ...
 
  // Now we use Adams-Bashforth method for all remaining steps
  while (t<tFinal) {

    // ...

    t += dt;

    // Add contribution to norms for error computation
    for (int i=0; i<N; i++) {
      double sol = exactSol(t,initCond(x[i]));
      normSol  += pow(sol,2);
      normDiff += // ...
    }
  
    // ...
      
  }
 
  std::cout << "dt: \t" << dt << " \t error: " << sqrt(normDiff/normSol) << std::endl;

  return 0;

}

